//***************************************************************************
//  x10.h
//  PSC05 x10 Interface Driver
//
//  Connect:
//           Any MCU         PSC05 Powerline Interface (Trans/Recieve)
//              A0              1          Zero Cross Detect
//              GND             2          Zero Cross Common
//              A1              3          Receive Output
//              A2              4          Transmit X10 Data
//
//           Any MCU          PSC04 Powerline Interface  (Transmit Only)
//              A0              1          Zero Cross Detect
//              Gnd             2          Zero Cross Common
//              Gnd             3          X10 Common
//              A1              4          Transmit X10 Signal
//***************************************************************************
// Program Syntax
//
// Display_X10(LCD Row, House Column, Channel Column, Status Column);
// x10_read()  Returns house_code, key_code(chan), key_code(status)
// x10Trans(House(Chr 'A' - 'P', Chan (0 - 15), On/Off(20 = ON, 28 = OFF))
//***************************************************************************
#define Zero_Crossing   PORTC.F0   // Receive Zero Crossing from Power Line
#define TRANS_TO_X10    PORTC.F1   // Transmit to X10
#define RCV_FROM_X10    PORTC.F2   // Receive from X10

char const House_Code_Table[17] = {'M','N','O','P','C','D','A','B','E',
                                  'F','G','H','K','L','I','J'};
char const Key_Code_Table[17] = {13, 14, 15, 16, 3, 4, 1, 2, 5, 6, 7,
                                  8, 11, 12, 9, 10};
char txt4[4], temp, setbit;
char house_code, key_code;
//***************************************************************************
char shiftRight(char number)
{
        return number >>= 1;
}
//***************************************************************************
char Rcv_Data()
{
  return (!RCV_FROM_X10);
}
//***************************************************************************
void wait_for_zero_crossing()
{
  if(Zero_Crossing)
     while(Zero_Crossing) ;
  else
     while(!Zero_Crossing) ;
}
//***************************************************************************
void x10_write_bits(char datam, char numofbits, char startbit)
{
   char loopcount;

   for(loopcount = 1; loopcount <= numofbits; ++loopcount)
     {
     wait_for_zero_crossing();
     datam = shiftRight(datam);
     TRANS_TO_X10 = datam.F0;
     delay_ms(1);
     TRANS_TO_X10 = 0;
     if(startbit == 0)
        {
        wait_for_zero_crossing();
        TRANS_TO_X10 = !datam.F0;
        delay_ms(1);
        TRANS_TO_X10 = 0;
        }
     }
     return;
}
//***************************************************************************
void x10_write(char house_code2, char key_code2)
{
  char i;
  i = 0;
  while (House_Code_Table[i] != house_code2)
    i++;
  house_code2 = i;
  if(key_code2 < 16)
     {
     i = 0;
     while (Key_Code_Table[i] != key_code2)
       i++;
     key_code2 = i;
     }
  x10_write_bits(7 * 2, 4, 1);
  x10_write_bits(house_code2 * 2, 4, 0);
  x10_write_bits(key_code2 * 2, 5, 0);
  x10_write_bits(0, 6, 1);
  return;
}
//***************************************************************************
char x10_read_bits(char num)
{
char ic, dataval;

   for(ic = 1; ic <= num; ++ic)
      {
      wait_for_zero_crossing();
      delay_us(150);
      if (RCV_FROM_X10 == 1)
         {
         setbit = 128;
         }
      else
         {
         setbit = 0;
         }
      dataval = dataval + setbit;
      dataval = shiftRight(dataval);
      wait_for_zero_crossing();
      delay_us(150);
   }
   for(ic = 0; ic < 7 - num; ++ic)
      {
      dataval = shiftRight(dataval);
      }
   return(dataval);
}
//***************************************************************************
void x10_read()   //(char house_code, char key_code)
{
  x10_read_bits(2);
  house_code = x10_read_bits(4);
  house_code = House_Code_Table[house_code];
  key_code   = x10_read_bits(5);
  if(key_code < 16)
    key_code = Key_Code_Table[key_code];
}
//****************************************************************************
void x10Trans(char House, char Chan, char OnOff)
{
      x10_write(House, Chan);       // Send House and Channel First
      x10_write(House, Chan);       // Must Transmit Data Twice
      Delay_ms(1000);
      x10_write(House, OnOff);      // Send House and Key Code Next
      x10_write(House, OnOff);      // Must Transmit Data Twice
}
//***************************************************************************
void Display_X10(char row, char HousePos, char ChanPos, char StatusPos)
{
      if(Rcv_Data())
        {
        x10_read();                           // Read Pulses from PSC05

        if (key_code < 19)                    // If valid Chan Number
           {
           temp = key_code;
           }

        LCD_Chr(row, HousePos, house_code);            // Display House Code
        BytetoStr(temp, txt4);
        LCD_Out(row, ChanPos, txt4);
        if (key_code == 20) LCD_Out(row, StatusPos, "ON "); // Display ON
        if (key_code == 28) LCD_Out(row, StatusPos, "OFF"); // Display OFF
        Delay_ms(1000);
        }
}
//****************************************************************************
void Display_X10_TFT()
{
      TFT_Set_Font(Tahoma19x19_Bold, CL_WHITE, FO_HORIZONTAL);
      if(Rcv_Data())
        {
        x10_read();                           // Read Pulses from PSC05

        if (key_code < 19)                    // If valid Chan Number
           {
           temp = key_code;
           }
        TFT_Set_Pen(CL_BLACK, 1);   // Writes over Char Field with Black
        TFT_Set_Brush(1, CL_BLACK, 0, TOP_TO_BOTTOM, CL_BLACK, CL_BLACK);
        TFT_Rectangle(20, 220, 140, 240);
        TFT_Write_Char(house_code, 20, 220);  // Display on Bottom "House Code"
        BytetoStr(temp, txt4);
        TFT_Write_Text(txt4, 50, 220);        // Display Chan
        if (key_code == 20) TFT_Write_Text("ON ", 90, 220);  // Display Status
        if (key_code == 28) TFT_Write_Text("OFF", 90, 220);
        Delay_ms(1000);
        }
}
//***************************************************************************